from sklearn.cluster import AgglomerativeClustering


def cluster(df, cols, distance_threshold=0.2, linkage='ward'):
    train_data = df[cols]
    agglo = AgglomerativeClustering(distance_threshold=distance_threshold, n_clusters=None, linkage=linkage)
    agglo.fit(train_data)
    df['labels'] = [str(l) for l in agglo.labels_]
    print(f"{len(df['labels'].unique())} clusters at distance {distance_threshold}")
    return df

if __name__ == '__main__':
    from pathlib import Path
    from glycomass.mass_list import load_mass_list
    from glycomass.process_data import process_data, process_folder

    mass_list = load_mass_list()
    folder_path = str(Path(__file__).parents[1]) + '/data/01Mar22'
    cols = list(mass_list.keys())  # ['1', '2', '3', '4', '5', '6', '7', '8']
    distance_threshold = 0.2
    linkage = 'ward'

    prefix = ""
    df = process_folder(folder_path, mass_list, remove_prefix=prefix)
    df = cluster(df, cols, distance_threshold=distance_threshold, linkage=linkage)

    Path(f"{folder_path}/result").mkdir(parents=True, exist_ok=True)
    df.to_excel(f"{folder_path}/result/processed_results.xlsx")



import itertools

def get_well_replacement_dict():
    well_rows = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H']
    well_cols = ['1', '2', '3', '4', '5', '6', '7', '8', '9']
    wells_replace_cols = ['01', '02', '03', '04', '05', '06', '07', '08', '09']
    wells = [f"{w[0]}{w[1]}" for w in list(itertools.product(well_rows, well_cols))]
    wells_replace = [f"{w[0]}{w[1]}" for w in list(itertools.product(well_rows, wells_replace_cols))]
    return {w: r for w, r in zip(wells, wells_replace)}

well_replacement_dict = get_well_replacement_dict()

def well_convert(name, spacer='_'):
    '''Converts any wells in name to A01 format (from A1)'''
    for well, replacement in well_replacement_dict.items():
        well = f"{well}{spacer}"
        if well in name:
            replacement = f"{replacement}{spacer}"
            name = name.replace(well, replacement)
    return name

if __name__ == '__main__':
    test_name = 'this_is_a_test_A1'
    new_name = well_convert(test_name)
    print(new_name)


import pandas as pd
import numpy as np
from pathlib import Path
from glycomass.well_replacement import well_convert

def load_data(filepath):
    dict_of_dfs = pd.read_excel(filepath,
                                sheet_name=None,
                                skiprows=[0,1],
                                header=[0])
    return dict_of_dfs

def get_intensities(df, mass_list, mz_col='m/z', int_col='Intens.'):
    intensity_dict = {}
    for dp, mass_range in mass_list.items():
        df_range = df[(df[mz_col] >= mass_range[0]) & (df[mz_col] <= mass_range[1])]
        intensity = df_range[int_col].max()
        if intensity is np.nan:
            intensity = 0
        intensity_dict[dp] = intensity

    return intensity_dict

def get_intensity_ratios(intensity_dict, round_to=4):
    sum_intensity = sum(list(intensity_dict.values()))

    ratio_intensity_dict = {}
    for key, value in intensity_dict.items():
        ratio_intensity_dict[key] = round(value/sum_intensity, round_to)

    return ratio_intensity_dict

def process_data(filepath, mass_list, remove_prefix='', convert_well_names=True):
    dict_of_dfs = load_data(filepath)
    list_results_dicts = []
    for name, df in dict_of_dfs.items():
        intensity_dict = get_intensities(df, mass_list)
        ratio_intensity_dict = get_intensity_ratios(intensity_dict)
        name = name.replace(remove_prefix, '')
        if convert_well_names is True:
            name = well_convert(name)
        ratio_intensity_dict['name'] = name
        list_results_dicts.append(ratio_intensity_dict)

    final_df = pd.DataFrame(list_results_dicts)

    # reorder cols so name is first
    cols = cols = final_df.columns.tolist()
    cols = cols[-1:] + cols[:-1]
    final_df = final_df[cols]
    return final_df

def process_folder(folder_path, mass_list, remove_prefix='', convert_well_names=True):

    pathlist = Path(folder_path).glob('*.xlsx')
    list_dfs = []
    for filepath in pathlist:
        df = process_data(filepath, mass_list, remove_prefix=remove_prefix, convert_well_names=convert_well_names)
        list_dfs.append(df)

    final_df = pd.concat(list_dfs)
    final_df = final_df.reindex()
    return final_df


if __name__ == '__main__':
    from glycomass.mass_list import load_mass_list

    mass_list = load_mass_list()
    filepath = str(Path(__file__).parents[0]) + '/data/test_data.xlsx'
    prefix = "Itag_assay_spot_"
    df = process_data(filepath, mass_list, remove_prefix=prefix)

    # save and print the test
    print(df.head())
    df.to_excel(str(Path(__file__).parents[0]) + '/data/test_results.xlsx')


    folder_path =  str(Path(__file__).parents[0]) + '/data/test_run'
    df = process_folder(folder_path, mass_list)
    print(df.head())
    Path(str(Path(__file__).parents[0]) + '/data/test_run/result').mkdir(parents=True, exist_ok=True)
    df.to_excel(str(Path(__file__).parents[0]) + '/data/test_run/result/test_results.xlsx')



import pandas as pd
from pathlib import Path

default_mass_list = f'{str(Path(__file__).parents[0])}/mass_list.xlsx'

def load_mass_list(filename=default_mass_list):
    df = pd.read_excel(filename)
    mass_list = {}
    for index, row in df.iterrows():
        mass_list[str(row['dp'])] = [row['low'], row['high']]
    return mass_list

if __name__ =='__main__':
    mass_list = load_mass_list()
    print(mass_list)



from glycomass.mass_list import load_mass_list
from glycomass.process_data import process_folder
from glycomass.clustering import cluster
from pathlib import Path

mass_list = load_mass_list()
cols = list(mass_list.keys())

def run(folder_path, cols=cols, remove_prefix="", linkage='ward', distance_threshold=0.2, convert_well_names=True):
    df = process_folder(folder_path, mass_list, remove_prefix=remove_prefix, convert_well_names=convert_well_names)
    df = cluster(df, cols, distance_threshold=distance_threshold, linkage=linkage)
    Path(f"{folder_path}/result").mkdir(parents=True, exist_ok=True)
    df.to_excel(f"{folder_path}/result/processed_results.xlsx")

if __name__ == '__main__':
    folder_path = str(Path(__file__).parents[1]) + '/data/01Mar22'
    run(folder_path)




from setuptools import setup, find_packages
from glycomass import __version__

with open('requirements.txt') as f:
  requirements = f.read().splitlines()

setup(
  name = 'glycomass',
  packages = find_packages(),
  include_package_data=True,
  version = __version__,
  license='',
  description = 'Sugar analysis',
  author = 'William Finnigan',
  author_email = 'wjafinnigan@gmail.com',
  url = '',
  download_url = '',
  keywords = ['sugar'],
  install_requires=requirements,
  classifiers=[
    'Development Status :: 3 - Alpha',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3'],
)